@@ -1,7 +1,7 @@
 from __future__ import absolute_import, division, print_function
 
 from multipledispatch import MDNotImplementedError
-from into import Chunks, chunks, convert, discover
+from into import Chunks, chunks, convert, discover, into
 from collections import Iterator, Iterable
 from toolz import curry, concat, map
 from datashape.dispatch import dispatch
@@ -44,3 +44,14 @@ def compute_down(expr, data, map=map, **kwargs):
         intermediate = list(concat(parts))
 
     return compute(agg_expr, {agg: intermediate})
+
+
+Cheap = (Head, ElemWise, Distinct, Symbol)
+
+@dispatch(Head, Chunks)
+def compute_down(expr, data, **kwargs):
+    leaf = expr._leaves()[0]
+    if all(isinstance(e, Cheap) for e in path(expr, leaf)):
+        return compute(expr, {leaf: into(Iterator, data)}, **kwargs)
+    else:
+        raise MDNotImplementedError()
