@@ -19,10 +19,13 @@ import operator
 import re
 from collections import Iterator
 from functools import partial
-from toolz import map, filter, compose, juxt, identity
-from cytoolz import groupby, reduceby, unique, take, concat, first, nth, pluck
+import toolz
+from toolz import map, filter, compose, juxt, identity, tail
+try:
+    from cytoolz import groupby, reduceby, unique, take, concat, nth, pluck
+except ImportError:
+    from toolz import groupby, reduceby, unique, take, concat, nth, pluck
 import datetime
-import cytoolz
 import toolz
 import math
 from datashape.predicates import isscalar, iscollection
@@ -31,7 +34,7 @@ from ..dispatch import dispatch
 from ..expr import (Projection, Field, Broadcast, Map, Label, ReLabel,
                     Merge, Join, Selection, Reduction, Distinct,
                     By, Sort, Head, Apply, Summary, Like,
-                    DateTime, Date, Time, Millisecond, symbol, ElemWise,
+                    DateTime, Date, Time, Millisecond, ElemWise, symbol,
                     Symbol, Slice, Expr, Arithmetic, ndim, DateTimeTruncate,
                     UTCFromTimestamp)
 from ..expr import reductions
@@ -42,7 +45,8 @@ from ..compatibility import builtins, apply, unicode, _inttypes
 from .core import compute, compute_up, optimize, base
 
 from ..utils import listpack
-from .pyfunc import lambdify, broadcast_collect
+from ..expr.broadcast import broadcast_collect
+from .pyfunc import lambdify
 from . import pydatetime
 
 # Dump exp, log, sin, ... into namespace
@@ -54,6 +58,7 @@ __all__ = ['compute', 'compute_up', 'Sequence', 'rowfunc', 'rrowfunc']
 
 Sequence = (tuple, list, Iterator, type(dict().items()))
 
+
 @dispatch(Expr, Sequence)
 def pre_compute(expr, seq, scope=None, **kwargs):
     try:
@@ -126,9 +131,9 @@ def rowfunc(t):
     See Also:
         compute<Rowwise, Sequence>
     """
-    from cytoolz.curried import get
+    from toolz.itertoolz import getter
     indices = [t._child.fields.index(col) for col in t.fields]
-    return get(indices)
+    return getter(indices)
 
 
 @dispatch(Field)
@@ -146,6 +151,7 @@ def rowfunc(t):
 def rowfunc(expr):
     return eval(funcstr(expr))
 
+
 @dispatch(Map)
 def rowfunc(t):
     if isscalar(t._child.dshape.measure):
@@ -163,10 +169,12 @@ def rowfunc(t):
 def rowfunc(t):
     return lambda row: getattr(row, t.attr)
 
+
 @dispatch(UTCFromTimestamp)
 def rowfunc(t):
     return datetime.datetime.utcfromtimestamp
 
+
 @dispatch((Date, Time))
 def rowfunc(t):
     return lambda row: getattr(row, t.attr)()
@@ -186,14 +194,17 @@ def rowfunc(expr):
 def rowfunc(expr):
     return getattr(math, type(expr).__name__)
 
+
 @dispatch(USub)
 def rowfunc(expr):
     return operator.neg
 
+
 @dispatch(Not)
 def rowfunc(expr):
     return operator.invert
 
+
 @dispatch(Arithmetic)
 def rowfunc(expr):
     if not isinstance(expr.lhs, Expr):
@@ -262,6 +273,7 @@ def compute_up(t, seq, **kwargs):
     else:
         return func(seq)
 
+
 @dispatch(Broadcast, Sequence)
 def compute_up(t, seq, **kwargs):
     func = rowfunc(t)
@@ -312,7 +324,7 @@ def compute_up_1d(expr, seq, **kwargs):
     try:
         return len(seq)
     except TypeError:
-        return cytoolz.count(seq)
+        return toolz.count(seq)
 
 
 @dispatch(ElemWise, base)
@@ -362,13 +374,13 @@ def _std(seq, unbiased):
 
 @dispatch(count, Sequence)
 def compute_up_1d(t, seq, **kwargs):
-    return cytoolz.count(filter(None, seq))
+    return toolz.count(filter(None, seq))
 
 
 @dispatch(Distinct, Sequence)
 def compute_up(t, seq, **kwargs):
     try:
-        row = first(seq)
+        row = toolz.first(seq)
     except StopIteration:
         return ()
     seq = concat([[row], seq]) # re-add row to seq
@@ -426,6 +438,7 @@ def child(expr):
         raise ValueError()
     return expr._inputs[0]
 
+
 def reduce_by_funcs(t):
     """ Create grouping func and binary operator for a by-reduction/summary
 
@@ -508,7 +521,10 @@ def pair_assemble(t):
 
     This is mindful to shared columns as well as missing records
     """
-    from cytoolz import get  # not curried version
+    try:
+        from cytoolz import get  # not curried version
+    except:
+        from toolz import get
     on_left = [t.lhs.fields.index(col) for col in listpack(t.on_left)]
     on_right = [t.rhs.fields.index(col) for col in listpack(t.on_right)]
 
@@ -646,9 +662,20 @@ def compute_up(expr, seq, **kwargs):
     if isinstance(index, tuple) and len(index) == 1:
         index = index[0]
     if isinstance(index, _inttypes):
-        return nth(index, seq)
+        try:
+            return seq[index]
+        except:
+            if index >= 0:
+                return nth(index, seq)
+            else:
+                return tail(-index, seq)[0]
     if isinstance(index, slice):
-        return itertools.islice(seq, index.start, index.stop, index.step)
+        if (index.start and index.start < 0 and
+            index.stop is None and
+            index.step in (1, None)):
+            return tail(-index.start, seq)
+        else:
+            return itertools.islice(seq, index.start, index.stop, index.step)
     raise NotImplementedError("Only 1d slices supported")
 
 
