@@ -48,12 +48,6 @@ class Node(object):
     def _inputs(self):
         return tuple([getattr(self, i) for i in self.__inputs__])
 
-    def __nonzero__(self): # pragma: no cover
-        return True
-
-    def __bool__(self):
-        return True
-
     def _leaves(self):
         """ Leaves of an expresion tree
 
@@ -136,9 +130,11 @@ class Node(object):
         ident = self.isidentical(other)
         if ident is True:
             return ident
+
         try:
             return self._eq(other)
-        except:
+        except AttributeError:
+            # e.g., we can't compare whole tables to other things (yet?)
             pass
         return False
 
@@ -220,6 +216,10 @@ class Node(object):
     def __invert__(self):
         return self._invert()
 
+    def __abs__(self):
+        from .math import abs
+        return abs(self)
+
 
 def isidentical(a, b):
     """ Strict equality testing
@@ -239,6 +239,12 @@ def isidentical(a, b):
 
     >>> isidentical(x + 1, x + 2)
     False
+
+    >>> isidentical((x, x + 1), (x, x + 1))
+    True
+
+    >>> isidentical((x, x + 1), (x, x + 2))
+    False
     """
     if isinstance(a, base) and isinstance(b, base):
         return a == b
@@ -246,6 +252,8 @@ def isidentical(a, b):
         return False
     if isinstance(a, Node):
         return all(map(isidentical, a._args, b._args))
+    if isinstance(a, (list, tuple)) and isinstance(b, (list, tuple)):
+        return len(a) == len(b) and all(map(isidentical, a, b))
     return a == b
 
 
@@ -361,7 +369,7 @@ def path(a, b):
         if not a._inputs:
             break
         for child in a._inputs:
-            if b in child._traverse():
+            if any(b.isidentical(node) for node in child._traverse()):
                 a = child
                 break
     yield a
